#!/usr/bin/perl -w
#
# Name        : check-jmol-missing-projects.pl
# Version     : 1.0
# License     : GPL
# Author      : Bas Couwenberg <sebastic@xs4all.nl>
# Description : Download the JMOL missing projects list and check if a 
#               directory holds a xyz file for one or more of those projects.
#               If we have one or more xyz files for missing projects in the 
#               list email the files to Nicolas Vervelle
# 

use strict;
use File::Basename;
use LWP::UserAgent;
use MIME::Lite;
use Getopt::Mixed "nextOption";

$|=1;

my %cfg = ( self            => 'check-jmol-missing-projects.pl',
            version         => '1.0',

            'dir'           => '/var/www/folding/mod_perl/client-data/',
            
	    'url'           => 'http://jmol.sourceforge.net/fah/missingProjects.txt',
	    'missing-file'  => dirname($0).'/missingProjects.txt',
	    'sent-file'     => dirname($0).'/sentProjects.txt',
	    
	    'email-to'      => 'Nicolas Vervelle <nicove@users.sourceforge.net>',
	    'email-from'    => 'Default Name <username@example.com>',
	    'email-subject' => '.xyz file for missing F@H project on JMOL website',
	    
	    download        => 1,
	    test            => 0,
            verbose         => 0,
          );

# get commandline parameters
Getopt::Mixed::init("dir=s
                     missing-file=s
		     sent-file=s
		     email-from=s
		     email-to=s
		     email-subject=s
		     no-download
		     force
		     test
		     verbose
		     help
		     d>dir
		     m>missing-file
		     s>sent-file
		     n>no-download
		     f>force
		     t>test
		     v>verbose
		     h>help
		    ");

while (my ($option, $value) = nextOption()) {
        if($option eq 'help') {
		print_usage(%cfg);
		exit;
	}
	
	$cfg{'dir'}           = $value if($option eq 'dir');
	$cfg{'missing-file'}  = $value if($option eq 'missing-file');
	$cfg{'sent-file'}     = $value if($option eq 'sent-file');
	$cfg{'email-from'}    = $value if($option eq 'email-from');
	$cfg{'email-to'}      = $value if($option eq 'email-to');
	$cfg{'email-subject'} = $value if($option eq 'email-subject');
        
	$cfg{'download'}      = 0      if($option eq 'no-download');
	$cfg{'force'}         = 1      if($option eq 'force');
	$cfg{'test'}          = 1      if($option eq 'test');
	$cfg{'verbose'}       = 1      if($option eq 'verbose');
}
Getopt::Mixed::cleanup();

# get the Modified HTTP header value from the first line of the file
my $modified_file = $cfg{'missing-file'}.'.modified';
my $last_modified = '';

print "Looking up Last-Modified HTTP header in: $modified_file ... " if($cfg{verbose});
if(-e $modified_file && -r $modified_file) {
	open(F, $modified_file) || die "Cannot open file: $modified_file ($!)";
	while(<F>) {
		chomp;
		$last_modified = $_;
		last;
	}
	close F;

	if($last_modified eq '') {
		print "Not found.\n" if($cfg{verbose});
	}
	else {
		print "Found.\nLast-Modified: $last_modified\n" if($cfg{verbose});
	}
}
else {
	if(!-e $modified_file) {
		print "File does not exist.\n" if($cfg{verbose});
	}
	elsif(!-r $modified_file) {
		print "Cannot read file!\n" if($cfg{verbose});
	}
	else {
		print "Unknown error!\n" if($cfg{verbose});
	}
}

# check the URL for modification, only download if we have an older version
if($last_modified !~ /^\s*$/ && $cfg{download}) {
	# download the file from the jmol website if it has been modified
	my $ua  = LWP::UserAgent->new();
	   $ua->agent("$cfg{self}/$cfg{version}");
   	
	print "Checking URL for modification: $cfg{url} ... " if($cfg{verbose});

	my $req = HTTP::Request->new( HEAD => $cfg{url},
	                              HTTP::Headers->new('If-Modified-Since' => $last_modified)
				    );
	my $res = $ua->request($req);
	if($res->code eq '304') {
		# the file was not modified, skip download & processing
		print "Not Modified.\n" if($cfg{verbose});
		
		if($cfg{force}) {
			# we will only download files if they were modified
			# or if we're forced to
			
			print "Parameter --force given, downloading anyway.\n" if($cfg{verbose});
		}
		else {
			if($cfg{verbose}) {
				print "Local copy is up-to-date, skipping download.\n";
			}
			$cfg{download} = 0;
		}
	}
	else {
		print "Modified!\n" if($cfg{verbose});
	}
}

# actually download the file
if($cfg{download} == 1) {
	my $ua  = LWP::UserAgent->new();
	   $ua->agent("$cfg{self}/$cfg{version}");
	   
	print "Downloading URL: $cfg{url} ... " if($cfg{verbose});
	
	my $req = HTTP::Request->new( GET => $cfg{url} );
	my $res = $ua->request($req);
	if($res->is_success) {
		print "Success\n" if($cfg{verbose});
		
		$last_modified = $res->header('Last-Modified');
		print "Last-Modified: $last_modified\n" if($cfg{verbose});
		
		print "Saving file to disk: $cfg{'missing-file'}\n" if($cfg{verbose});
		open(F, ">", $cfg{'missing-file'}) || die "Cannot open file: $cfg{'missing-file'} ($!)";
		print F $res->content;
		close F;

		print "Saving Last-Modified header to file: $modified_file\n" if($cfg{verbose});
		open(F, ">", $modified_file) || die "Cannot open file: $modified_file ($!)";
		print F $last_modified;
		close F;
	}
	else {
		print "Failed!\n" if($cfg{verbose});
		print "Error: Failed to retrieve URL! ($cfg{url})\n";
		print "Server returned: ".$res->code." ".$res->message."\n";
		
		exit 1;
	}
}

# for each project in the missingProjects file, 
# check our .xyz files to see if we have that project
if(-r $cfg{'missing-file'}) {
	my %xyz_of;
	my %sent_projects;
	my $found_projects = 0;
	
	# compile a list of the projects we've sent already
	if(-r $cfg{'sent-file'}) {
		print "Reading sent projects file: $cfg{'sent-file'}\n" if($cfg{verbose});
		
		open(F, $cfg{'sent-file'}) || die "Cannot open file: $cfg{'missing-file'} ($!)";
		while(<F>) {
			chomp;
			if(/^(\d+)/) {
				$sent_projects{$1} = $1;
			}
		}
		close F;
	}
	
	print "Checking xyz files in directory: $cfg{'dir'} ... \n" if($cfg{verbose});
	
	# for every project listed in the missing projects file
	# check the xyz. files in the directory if they have that project
	open(F, $cfg{'missing-file'}) || die "Cannot open file: $cfg{'missing-file'} ($!)";
	while(<F>) {
		chomp;
		if(/^(\d+)/) {
			my $project = $1;
			my $string  = 'p'.$project.'_';

			# skip this project if it is listed in the sent projects list
			if($sent_projects{$project}) {
				next;
			}
			
			opendir(DIR, $cfg{'dir'}) || die "Cannot open dir: $cfg{'dir'} ($!)";
			foreach(readdir DIR) {
				# if we have already found the .xyz file for this project
				# we can skip all the other files
				if($xyz_of{$project}) {
					last;
				}
				
				# xyz.hostname (FCI 0.x)
				# current.xyz  (F@H)
				if(/^xyz\./ || /\.xyz$/) {
					my $f    = $_;
					my $file = $cfg{'dir'}.$f;

					my $i = 0;
					open(DIRF, $file) || die "Cannot open file: $file ($!)";
					while(<DIRF>) {
						chomp;
						
						# two examples from actual current.xyz files:
						#   393  p1112_L939_K12M_nat_min1_355K
						#9130    p1150_RIBO_semihelixfrom1141
						if(/^\s*\d+\s+($string.*)\s*/) {
							chomp(my $name = $1);
							
							print " Project: $project\tFile: $f\n";
							
							$xyz_of{$project}{name} = $name;
							$xyz_of{$project}{file} = $file;

							$found_projects++;
						}
						last;
					}
					close DIRF;
				}
			}
			closedir DIR;
		}
	}
	close F;
	
	print "Found $found_projects of the missing projects.\n" if($cfg{verbose});
	if($found_projects > 0) {
		my $projects = '';
		foreach my $project (sort keys %xyz_of) {
			$projects .= " $project";
			$projects .= "\t$xyz_of{$project}{name}\n";
		}

		my $email_body  = "Hi,\n\n";
		   $email_body .= "This is an automatically generated message to send you one or more projects ";
		   $email_body .= "which were listed in the JMOL missing projects text file.\n";
		   $email_body .= "The missing projects file used was the one which was last modifed at: $last_modified\n";
		   $email_body .= "The project .xyz files which were found are:\n$projects\n";
		   $email_body .= "Regards,\n$cfg{'email-from'}\n";
		
		if($cfg{'test'}) {
			print "Not sending email to: $cfg{'email-to'}\n";
			print "Would have sent:\n\n";
			print "       From: $cfg{'email-from'}\n";
			print "         To: $cfg{'email-to'}\n";
			print "    Subject: $cfg{'email-subject'}\n";
			print "Attachments: ";

			my $i = 0;
			foreach my $project (sort keys %xyz_of) {
				print "             " if($i > 0);
				print 'p'. $project .".xyz\t($xyz_of{$project}{file})\n";
				$i++;
			}
			
			print "\n$email_body\n";
		}
		else {
			my $msg = MIME::Lite->new( From    => $cfg{'email-from'},
						   To      => $cfg{'email-to'},
						   Subject => $cfg{'email-subject'},
						   Type    => 'TEXT',
						   Data    => $email_body
						 );
		
			# attach the .xyz files
			foreach my $project (sort keys %xyz_of) {
				print "Attaching .xyz file for project $project ...\n";
	
				$msg->attach( Type        => 'text/plain',
				              Path        => $xyz_of{$project}{file},
					      Filename    => 'p'. $project .'.xyz',
					      Disposition => 'attachment'
					    );
			
				# add the project to the sent projects list
				open(F, ">>", $cfg{'sent-file'}) || die "Cannot open file: $cfg{'missing-file'} ($!)";
				print F $project."\n";
				close F;
			}
		
			print "Sending email to: $cfg{'email-to'} ...\n";
			$msg->send || die $msg->err;
		}
	}
}
	
################################[ Subroutines ]################################

sub print_usage {
	my %cfg = @_;
	
	print "$cfg{self} v$cfg{version}\n\n";
	print "Usage: $cfg{self} [OPTIONS]\n\n";
	print "Options:\n";
	
	print "-d, --dir <DIRECTORY>      Path to the directory with xyz files.\n";
	print "-m, --missing-file <FILE>  Path to the missing projects file.\n";
	print "-s, --sent-file <FILE>     Path to the sent projects file.\n";
	print "-n, --no-download          Don't download the file, use the one on disk.\n";
	print "-f, --force                Force download of the file, even if it's up-to-date.\n";
	print "-t, --test                 Test run, don't send email print it to STDOUT.\n";
	print "    --email-from <EMAIL>   Email address of the sender.\n";
	print "    --email-to <EMAIL>     Email address of the recipient.\n";
	print "    --email-subject <TEXT> Email subject string.\n";
	print "-v, --verbose              Enable verbose output\n";
	print "-h, --help                 Print this usage information\n";
}
